'================
'ColorMyHeart.bas v1.1
'================
'A 'flush' like puzzle game for QB64
'Coded by Dav August/2023

'NEW for v1.1:  - Game Autosizes to fit users desktop.
'               - New, cleaner graphics
'               - Added game Sounds
'
'====
'GOAL:
'====
'
'The goal is to set all puzzle squares to its proper color.
'For example, squares with blue hearts should be colored blue.
'Squares with red heart squares should be colored red, etc...
'Clear squares (no hearts) should be clear white (no color).

'===========
'HOW TO PLAY:
'===========
'
'First pick a color TO use on the top row (it will be checked).
'Then press a direction arrow to put the color across a row or a
'column. The selected color will not pass through another color,
'only white squares. Also, colors will not go through an X square.

'Use the erase (white color) to set a square color back to white.
'It will only clear one square at a time.

'The CLS button is a level redo, it resets current level.

'When all squares are colored or cleared correctly, you will
'advance to the next level.  There are currently 15 levels.

'The current level is displayed in the title bar.

'Use can change levels using left/right arrow keys.

'ESC key quits game

'=============================================================


DIM SHARED grid, size: grid = 6: size = _DESKTOPHEIGHT * .85 / 7
DIM SHARED bv(grid * grid), bx(grid * grid), by(grid * grid)
DIM SHARED bv$(grid * grid), fc(grid * grid)

SCREEN _NEWIMAGE(size * grid, size * grid + size, 32)

'Load program icon
ttmp = _LOADIMAGE("icon.png", 32)
IF ttmp < -1 THEN
    _ICON ttmp: _FREEIMAGE ttmp
END IF

'set game defaults
level = 1: maxlevel = 15: fclr = 1


'====================
top: 'Start new level
'====================

_SNDPLAYFILE ("sfx_magic.ogg")

_TITLE "Color My Heart: Level " + STR$(level) + " of " + STR$(maxlevel)

'init button values, and x/y values...
bc = 1
FOR r = 1 TO grid
    FOR c = 1 TO grid
        x = (r * size): y = (c * size)
        IF bc = 1 THEN GOTO skip 'dont draw corners...
        IF bc = 6 THEN GOTO skip
        IF bc = 31 THEN GOTO skip
        IF bc = 36 THEN GOTO skip
        skip:
        bx(bc) = x - size: by(bc) = y: bv(bc) = 0
        bv$(bc) = STR$(bc): fc(bc) = 0
        bc = bc + 1
    NEXT
NEXT

'draw top arrows
FOR b = 7 TO 30 STEP grid
    Show "top.png", bx(b), by(b), 0
NEXT

'draw left arrows
FOR b = 2 TO 5
    Show "left.png", bx(b), by(b), 0
NEXT

'draw right arrows
FOR b = 32 TO 35
    Show "right.png", bx(b), by(b), 0
NEXT

'draw bottom arrows
FOR b = 12 TO 30 STEP grid
    Show "bottom.png", bx(b), by(b), 0
NEXT

'set squares filename data
xx$ = "block" 'the x box
blu$ = "bluclr": red$ = "redclr" 'colored hearts
grn$ = "grnclr": yel$ = "yelclr": clr$ = "white"

'draw color selection area
GOSUB colorbuttons

'set level data
GOSUB setlevel


'====== Main game loop ======

DO

    'Get a keypress
    Key$ = UCASE$(INKEY$)

    'right arrow, advance level
    IF Key$ = CHR$(0) + CHR$(77) THEN
        IF level < maxlevel THEN
            level = level + 1: GOTO top
        END IF
    END IF
    'left arrow, advance level
    IF Key$ = CHR$(0) + CHR$(75) THEN
        IF level > 1 THEN
            level = level - 1: GOTO top
        END IF
    END IF

    'ESC key quits
    IF Key$ = CHR$(27) THEN SYSTEM

    'wait until mouse button up to continue
    WHILE _MOUSEBUTTON(1) <> 0: n = _MOUSEINPUT: WEND

    trap = _MOUSEINPUT
    IF _MOUSEBUTTON(1) THEN
        mx = _MOUSEX: my = _MOUSEY

        ''==== if clicked color buttons
        IF mx < size AND my < size THEN
            fclr = 1: GOSUB colorbuttons: GOTO doneit
        END IF
        IF mx >= size AND mx < size * 2 AND my < size THEN
            fclr = 2: GOSUB colorbuttons: GOTO doneit
        END IF
        IF mx >= size AND mx < size * 3 AND my < size THEN
            fclr = 3: GOSUB colorbuttons: GOTO doneit
        END IF
        IF mx >= size AND mx < size * 4 AND my < size THEN
            fclr = 4: GOSUB colorbuttons: GOTO doneit
        END IF
        IF mx >= size AND mx < size * 5 AND my < size THEN
            fclr = 0: GOSUB colorbuttons: GOTO doneit
        END IF
        IF mx >= size AND mx < size * 6 AND my < size THEN
            Show "clsselect.png", size * 5, 0, .1
            GOTO top
        END IF

        '========================= handle top & bottom arrows now....
        'handle 1st row down
        IF mx >= size AND mx < size * 2 AND my >= size AND my < size * 2 THEN
            Show "topselect.png", size, size, .1: Show "top.png", size, size, 0
            FOR s = 8 TO 11
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 1st row up
        IF mx >= size AND mx < size * 2 AND my >= (size * 6) AND my < (size * 6) + size THEN
            Show "bottomselect.png", size, size * 6, .1: Show "bottom.png", size, size * 6, 0
            FOR s = 11 TO 8 STEP -1
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 2nd row down
        IF mx >= size * 2 AND mx < size * 3 AND my >= size AND my < size * 2 THEN
            Show "topselect.png", size * 2, size, .1: Show "top.png", size * 2, size, 0
            FOR s = 14 TO 17
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 2nd row up
        IF mx >= size * 2 AND mx < size * 3 AND my >= (size * 6) AND my < (size * 6) + size THEN
            Show "bottomselect.png", size * 2, size * 6, .1: Show "bottom.png", size * 2, size * 6, 0
            FOR s = 17 TO 14 STEP -1
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 3rd row down
        IF mx >= size * 3 AND mx < size * 4 AND my >= size AND my < size * 2 THEN
            Show "topselect.png", size * 3, size, .1: Show "top.png", size * 3, size, 0
            FOR s = 20 TO 23
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 3rd row up
        IF mx >= size * 3 AND mx < size * 4 AND my >= (size * 6) AND my < (size * 6) + size THEN
            Show "bottomselect.png", size * 3, size * 6, .1: Show "bottom.png", size * 3, size * 6, 0
            FOR s = 23 TO 20 STEP -1
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 4th row down
        IF mx >= size * 4 AND mx < size * 5 AND my >= size AND my < size * 2 THEN
            Show "topselect.png", size * 4, size, .1: Show "top.png", size * 4, size, 0
            FOR s = 26 TO 29
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 4th row up
        IF mx >= size * 4 AND mx < size * 5 AND my >= (size * 6) AND my < (size * 6) + size THEN
            Show "bottomselect.png", size * 4, size * 6, .1: Show "bottom.png", size * 4, size * 6, 0
            FOR s = 29 TO 26 STEP -1
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF

        '========================= handle left & right arrows now....
        'handle 1st row going right
        IF mx >= 0 AND mx < size AND my >= size * 2 AND my < size * 3 THEN
            Show "leftselect.png", 0, size * 2, .1: Show "left.png", 0, size * 2, 0
            FOR s = 8 TO 26 STEP grid
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 1st row going left
        IF mx >= size * 5 AND mx < size * 6 AND my >= size * 2 AND my < size * 3 THEN
            Show "rightselect.png", size * 5, size * 2, .1: Show "right.png", size * 5, size * 2, 0
            FOR s = 26 TO 8 STEP -grid
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 2nd row going right
        IF mx >= 0 AND mx < size AND my >= size * 3 AND my < size * 4 THEN
            Show "leftselect.png", 0, size * 3, .1: Show "left.png", 0, size * 3, 0
            FOR s = 9 TO 27 STEP grid
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 2nd row going left
        IF mx >= size * 5 AND mx < size * 6 AND my >= size * 3 AND my < size * 4 THEN
            Show "rightselect.png", size * 5, size * 3, .1: Show "right.png", size * 5, size * 3, 0
            FOR s = 27 TO 9 STEP -grid
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 3rd row going right
        IF mx >= 0 AND mx < size AND my >= size * 4 AND my < size * 5 THEN
            Show "leftselect.png", 0, size * 4, .1: Show "left.png", 0, size * 4, 0
            FOR s = 10 TO 28 STEP grid
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 3rd row going left
        IF mx >= size * 5 AND mx < size * 6 AND my >= size * 4 AND my < size * 5 THEN
            Show "rightselect.png", size * 5, size * 4, .1: Show "right.png", size * 5, size * 4, 0
            FOR s = 28 TO 10 STEP -grid
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 4th row going right
        IF mx >= 0 AND mx < size AND my >= size * 5 AND my < size * 6 THEN
            Show "leftselect.png", 0, size * 5, .1: Show "left.png", 0, size * 5, 0
            FOR s = 11 TO 29 STEP grid
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF
        'handle 4th row going left
        IF mx >= size * 5 AND mx < size * 6 AND my >= size * 5 AND my < size * 6 THEN
            Show "rightselect.png", size * 5, size * 5, .1: Show "right.png", size * 5, size * 5, 0
            FOR s = 29 TO 11 STEP -grid
                GOSUB colorlogic: IF fclr > 0 THEN _SNDPLAYFILE ("sfx_pop2.wav")
            NEXT
        END IF

    END IF

    doneit:

    GOSUB checkforwin

LOOP


END



'=========================================================
'                  GOSUB ROUTINES
'=========================================================

'============
colorbuttons: 'draw color selection buttons
'============

_SNDPLAYFILE ("sfx_pop1.wav"):

IF fclr = 1 THEN Show "redcheck.png", 0, 0, 0 ELSE Show "red.png", 0, 0, 0
IF fclr = 2 THEN Show "greencheck.png", size, 0, 0 ELSE Show "green.png", size, 0, 0
IF fclr = 3 THEN Show "bluecheck.png", size * 2, 0, 0 ELSE Show "blue.png", size * 2, 0, 0
IF fclr = 4 THEN Show "yellowcheck.png", size * 3, 0, 0 ELSE Show "yellow.png", size * 3, 0, 0
IF fclr = 0 THEN Show "whitecheck.png", size * 4, 0, 0 ELSE Show "white.png", size * 4, 0, 0

'cls button
Show "cls.png", size * 5, 0, 0

'notes set "120:sg7"!notes play

RETURN


'=========
colorlogic: 'determins colors and draws them
'=========

IF fclr = 0 THEN _SNDPLAYFILE ("sfx_fill.ogg")

IF bv$(s) = xx$ THEN GOTO doneit 'hit an X block, so stop...

IF fclr <> 0 THEN
    IF fc(s) <> 0 THEN GOTO doneit
END IF

once = fc(s)

SELECT CASE fclr
    CASE IS = 0: tag$ = "clr"
    CASE IS = 1: tag$ = "red"
    CASE IS = 2: tag$ = "grn"
    CASE IS = 3: tag$ = "blu"
    CASE IS = 4: tag$ = "yel"
END SELECT

n$ = LEFT$(bv$(s), 3) + tag$

IF bv$(s) = "white" OR bv$(s) = "blue" OR bv$(s) = "red" OR bv$(s) = "green" OR bv$(s) = "yellow" THEN
    SELECT CASE fclr
        CASE IS = 0: n$ = "white"
        CASE IS = 1: n$ = "red"
        CASE IS = 2: n$ = "green"
        CASE IS = 3: n$ = "blue"
        CASE IS = 4: n$ = "yellow"
    END SELECT
END IF

Show n$ + ".png", bx(s), by(s), .05

fc(s) = fclr: IF fclr = 0 AND once <> 0 THEN GOTO doneit

RETURN


'===========
checkforwin: 'see if board colored correctly
'===========

'scan for completion
FOR z = 8 TO 29
    IF bv$(z) = clr$ THEN
        IF fc(z) <> 0 THEN GOTO breakout
    END IF
    IF bv$(z) = red$ THEN
        IF fc(z) <> 1 THEN GOTO breakout
    END IF
    IF bv$(z) = grn$ THEN
        IF fc(z) <> 2 THEN GOTO breakout
    END IF
    IF bv$(z) = blu$ THEN
        IF fc(z) <> 3 THEN GOTO breakout
    END IF
    IF bv$(z) = yel$ THEN
        IF fc(z) <> 4 THEN GOTO breakout
    END IF
NEXT z

_DELAY 1

_SNDPLAYFILE ("sfx_clap.ogg")

'draw ribbon hearts
ttmp = _LOADIMAGE("heart.png")
FOR f = 1 TO grid * grid
    IF f > 7 AND f < 12 THEN _PUTIMAGE (bx(f), by(f))-(bx(f) + size, by(f) + size), ttmp
    IF f > 13 AND f < 18 THEN _PUTIMAGE (bx(f), by(f))-(bx(f) + size, by(f) + size), ttmp
    IF f > 19 AND f < 24 THEN _PUTIMAGE (bx(f), by(f))-(bx(f) + size, by(f) + size), ttmp
    IF f > 25 AND f < 30 THEN _PUTIMAGE (bx(f), by(f))-(bx(f) + size, by(f) + size), ttmp
    _DELAY .03
NEXT f
_FREEIMAGE ttmp

_DELAY 1

IF level = maxlevel THEN
    CLS: END
END IF

level = level + 1

GOTO top

breakout:

RETURN



'=======
setlevel: 'put current level data into variables
'=======

IF level = 1 THEN
    bv$(8) = blu$: bv$(9) = blu$: bv$(10) = clr$: bv$(11) = blu$
    bv$(14) = xx$: bv$(15) = xx$: bv$(16) = yel$: bv$(17) = xx$
    bv$(20) = xx$: bv$(21) = red$: bv$(22) = xx$: bv$(23) = xx$
    bv$(26) = grn$: bv$(27) = clr$: bv$(28) = grn$: bv$(29) = grn$
END IF
IF level = 2 THEN
    bv$(8) = xx$: bv$(9) = clr$: bv$(10) = grn$: bv$(11) = clr$
    bv$(14) = clr$: bv$(15) = xx$: bv$(16) = clr$: bv$(17) = blu$
    bv$(20) = red$: bv$(21) = clr$: bv$(22) = xx$: bv$(23) = clr$
    bv$(26) = clr$: bv$(27) = yel$: bv$(28) = clr$: bv$(29) = xx$
END IF
IF level = 3 THEN
    bv$(8) = xx$: bv$(9) = xx$: bv$(10) = blu$: bv$(11) = xx$
    bv$(14) = red$: bv$(15) = yel$: bv$(16) = grn$: bv$(17) = xx$
    bv$(20) = xx$: bv$(21) = grn$: bv$(22) = yel$: bv$(23) = red$
    bv$(26) = xx$: bv$(27) = blu$: bv$(28) = xx$: bv$(29) = xx$
END IF
IF level = 4 THEN
    bv$(8) = xx$: bv$(9) = xx$: bv$(10) = blu$: bv$(11) = blu$
    bv$(14) = red$: bv$(15) = clr$: bv$(16) = yel$: bv$(17) = red$
    bv$(20) = red$: bv$(21) = yel$: bv$(22) = clr$: bv$(23) = red$
    bv$(26) = blu$: bv$(27) = blu$: bv$(28) = xx$: bv$(29) = xx$
END IF
IF level = 5 THEN
    bv$(8) = xx$: bv$(9) = red$: bv$(10) = red$: bv$(11) = red$
    bv$(14) = grn$: bv$(15) = grn$: bv$(16) = clr$: bv$(17) = xx$
    bv$(20) = xx$: bv$(21) = clr$: bv$(22) = blu$: bv$(23) = blu$
    bv$(26) = yel$: bv$(27) = yel$: bv$(28) = yel$: bv$(29) = xx$
END IF
IF level = 6 THEN
    bv$(8) = xx$: bv$(9) = grn$: bv$(10) = red$: bv$(11) = yel$
    bv$(14) = xx$: bv$(15) = clr$: bv$(16) = blu$: bv$(17) = red$
    bv$(20) = clr$: bv$(21) = blu$: bv$(22) = clr$: bv$(23) = grn$
    bv$(26) = yel$: bv$(27) = clr$: bv$(28) = xx$: bv$(29) = xx$
END IF
IF level = 7 THEN
    bv$(8) = xx$: bv$(9) = red$: bv$(10) = clr$: bv$(11) = grn$
    bv$(14) = blu$: bv$(15) = clr$: bv$(16) = xx$: bv$(17) = clr$
    bv$(20) = xx$: bv$(21) = grn$: bv$(22) = clr$: bv$(23) = red$
    bv$(26) = xx$: bv$(27) = xx$: bv$(28) = blu$: bv$(29) = xx$
END IF
IF level = 8 THEN
    bv$(8) = xx$: bv$(9) = clr$: bv$(10) = xx$: bv$(11) = xx$
    bv$(14) = yel$: bv$(15) = clr$: bv$(16) = clr$: bv$(17) = xx$
    bv$(20) = grn$: bv$(21) = clr$: bv$(22) = clr$: bv$(23) = xx$
    bv$(26) = xx$: bv$(27) = clr$: bv$(28) = xx$: bv$(29) = xx$
END IF
IF level = 9 THEN
    bv$(8) = clr$: bv$(9) = xx$: bv$(10) = blu$: bv$(11) = xx$
    bv$(14) = red$: bv$(15) = clr$: bv$(16) = grn$: bv$(17) = yel$
    bv$(20) = yel$: bv$(21) = grn$: bv$(22) = clr$: bv$(23) = red$
    bv$(26) = xx$: bv$(27) = blu$: bv$(28) = xx$: bv$(29) = clr$
END IF
IF level = 10 THEN
    bv$(8) = xx$: bv$(9) = yel$: bv$(10) = yel$: bv$(11) = xx$
    bv$(14) = red$: bv$(15) = clr$: bv$(16) = clr$: bv$(17) = grn$
    bv$(20) = red$: bv$(21) = clr$: bv$(22) = clr$: bv$(23) = grn$
    bv$(26) = xx$: bv$(27) = blu$: bv$(28) = blu$: bv$(29) = clr$
END IF
IF level = 11 THEN
    bv$(8) = xx$: bv$(9) = red$: bv$(10) = clr$: bv$(11) = xx$
    bv$(14) = red$: bv$(15) = clr$: bv$(16) = yel$: bv$(17) = yel$
    bv$(20) = clr$: bv$(21) = grn$: bv$(22) = clr$: bv$(23) = blu$
    bv$(26) = xx$: bv$(27) = grn$: bv$(28) = blu$: bv$(29) = clr$
END IF
IF level = 12 THEN
    bv$(8) = xx$: bv$(9) = blu$: bv$(10) = xx$: bv$(11) = xx$
    bv$(14) = blu$: bv$(15) = grn$: bv$(16) = clr$: bv$(17) = yel$
    bv$(20) = xx$: bv$(21) = clr$: bv$(22) = grn$: bv$(23) = clr$
    bv$(26) = xx$: bv$(27) = yel$: bv$(28) = clr$: bv$(29) = grn$
END IF
IF level = 13 THEN
    bv$(8) = xx$: bv$(9) = red$: bv$(10) = xx$: bv$(11) = grn$
    bv$(14) = red$: bv$(15) = clr$: bv$(16) = grn$: bv$(17) = clr$
    bv$(20) = clr$: bv$(21) = yel$: bv$(22) = clr$: bv$(23) = blu$
    bv$(26) = yel$: bv$(27) = xx$: bv$(28) = blu$: bv$(29) = xx$
END IF
IF level = 14 THEN
    bv$(8) = yel$: bv$(9) = grn$: bv$(10) = yel$: bv$(11) = grn$
    bv$(14) = grn$: bv$(15) = yel$: bv$(16) = grn$: bv$(17) = yel$
    bv$(20) = yel$: bv$(21) = grn$: bv$(22) = yel$: bv$(23) = grn$
    bv$(26) = grn$: bv$(27) = yel$: bv$(28) = grn$: bv$(29) = yel$
END IF
IF level = 15 THEN
    bv$(8) = xx$: bv$(9) = grn$: bv$(10) = grn$: bv$(11) = xx$
    bv$(14) = xx$: bv$(15) = red$: bv$(16) = clr$: bv$(17) = clr$
    bv$(20) = clr$: bv$(21) = clr$: bv$(22) = yel$: bv$(23) = blu$
    bv$(26) = xx$: bv$(27) = grn$: bv$(28) = grn$: bv$(29) = xx$
END IF

'draw button data
FOR s = 8 TO 11: Show bv$(s) + ".png", bx(s), by(s), .03: NEXT
FOR s = 14 TO 17: Show bv$(s) + ".png", bx(s), by(s), .03: NEXT
FOR s = 20 TO 23: Show bv$(s) + ".png", bx(s), by(s), .03: NEXT
FOR s = 26 TO 29: Show bv$(s) + ".png", bx(s), by(s), .03: NEXT

RETURN


'==============================================

SUB Show (nam$, x, y, dly)
    'Loads & puts image filename img$ on screen at x,y
    'dly is optional delay after putting image on screen
    'SUB frees up image handle after loading file.
    'This helper sub shortened the source alot.
    ttmp = _LOADIMAGE(nam$)
    _PUTIMAGE (x, y)-(x + size, y + size), ttmp: _FREEIMAGE ttmp
    IF dly <> 0 THEN _DELAY dly
END SUB



